var logLib = require('../lib/logger');
var configFinder = require('../lib/configurationInterpreter');
var textFieldType = require('../lib/textFieldType');

var pdfkit = require('pdfkit');
var bwipjs = require('bwip-js');
var async = require('async');

var log = logLib.logger('labelprint_pdfCreator');

function toPDFPoints(millimeters) {
	return millimeters * 2.83467;
}

/**
 * 
 * @param code
 * @param recognitionCode
 * @param callback
 */
function generateCode(code, recognitionCode, callback) {
	//illegal EAN13 special handling for <saw>
	if (code.bcid === 'ean13' && (recognitionCode.length !== 12 || recognitionCode.length !== 13)) {
		code.bcid = 'code128';
	}

	bwipjs.toBuffer({
		bcid: code.bcid,
		text: recognitionCode,
		scale: code.scale,
		height: code.height,
		includetext: code.includetext,
		textxalign: code.textalign,
		textsize: toPDFPoints(code.textsize)
	}, function (err, png) {
		callback(err, png);
	});
}

/**
 * 
 * @param sales
 * @param callback
 */
module.exports.create = function (sales, callback) {
	if (sales === null || sales.length === 0) {
		callback(null);
		return;
	}

	var config = configFinder.loadLabelLayout();

	var doc = new pdfkit({
		layout: config.layout,
		size: [toPDFPoints(config.width), toPDFPoints(config.height)],
		margins: {
			top: toPDFPoints(2),
			bottom: toPDFPoints(2),
			left: toPDFPoints(2),
			right: toPDFPoints(2)
		}
	});

	// metadata
	doc.info.Title = 'labels';
	doc.info.Author = 'KORONA.pos.labelprint';
	doc.info.Subject = 'labels';

	var isFirstPage = true;

	// sales
	async.eachSeries(sales, function (sale, saleCallback) {
		log.info(sale.description);

		if (!isFirstPage) {
			doc.addPage();
		} else {
			isFirstPage = false;
		}

		// textFields
		async.eachSeries(config.textFields, function (textField, textFieldCallback) {
			var content = textFieldType.content(textField.type, sale, textField);

			if (textField.hasOwnProperty('font')) {
				doc.font('public/fonts/' + textField.font);
			} else {
				doc.font('public/fonts/lato.medium.ttf');
			}

			doc.rotate(textField.rotate, {
				origin: [toPDFPoints(textField.x), toPDFPoints(textField.y)]
			})//
				.fontSize(toPDFPoints(textField.fontsize))//
				.text(content, toPDFPoints(textField.x), toPDFPoints(textField.y), {
					width: toPDFPoints(textField.width),
					height: toPDFPoints(textField.height)
				});

			// reset coordinates
			doc.rotate(-textField.rotate, {
				origin: [toPDFPoints(textField.x), toPDFPoints(textField.y)]
			});

			textFieldCallback(null);
		}, function doneProcessingTextFields(err) {
			if (err) {
				log.error(err);
			}
			log.debug('processed textFields of ' + sale.description);

			// codes
			async.eachSeries(config.codes, function (code, codeCallback) {
				generateCode(code, sale.recognitionCode, function (err, codePng) {
					if (err) {
						codeCallback(err);
						return;
					}
					doc.rotate(code.rotate, {
						origin: [toPDFPoints(code.x), toPDFPoints(code.y)]
					})//
						.image(codePng, toPDFPoints(code.x), toPDFPoints(code.y), {
							width: toPDFPoints(code.width),
							height: toPDFPoints(code.height)
						});

					// reset coordinates
					doc.rotate(-code.rotate, {
						origin: [toPDFPoints(code.x), toPDFPoints(code.y)]
					});

					codeCallback(null);
				});
			}, function doneProcessingCodes(err) {
				if (err) {
					log.error(err);
				}
				log.debug('processed codes of ' + sale.description);
				saleCallback(null);
			});

		});

	},

		// done
		function doneProcessingSales(err) {
			if (err) {
				log.error(err);
			}
			log.debug('processed sales');

			callback(doc);
		});
};